/*
 * $URL: https://repos.ser.asu.edu/svn/lroc/map_utils/trunk/src/calc_extents.c $
 * 
 * Copyright (C) 2012 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * This file has been released under the modified BSD license.
 * See COPYING in the distribution package for details.
 *
 * $Author: nme $
 *   $Date: 2015-10-29 08:18:29 -0700 (Thu, 29 Oct 2015) $
 *    $Rev: 16916 $
 *
 * Functions used by map_calc_extents
 */

#include "map_shared_code.h"

void run_extents(BBox range, double increment, BBox *coords, BBox *anticoords, BBox *extents, projPJ pj_dest, projPJ pj_latlon) {
	int ret=0;
	double lat,lon;

	for (double x=range.minx; x<=range.maxx; x += increment) {
	  for (double y=range.miny; y<=range.maxy; y += increment) {
	    if (x < -180 || x > 180 || y < -90 || y > 90) continue;
	    if (!pj_is_latlong(pj_dest)) {
	      lon = x*DEG_TO_RAD;
	      lat = y*DEG_TO_RAD;
	    } else {
	      lon = x;
	      lat = y;
	    };
	    ret = pj_transform(pj_latlon, pj_dest, 1, 1, &lon, &lat, NULL);
	    if (ret == 0) {
	      if (lon < extents->minx) {
		coords->minx = x;
		anticoords->minx = y;
		extents->minx = lon;
	      }
	      if (lon > extents->maxx) {
		coords->maxx = x;
		anticoords->maxx = y;
		extents->maxx = lon;
	      }
	      if (lat < extents->miny) {
		coords->miny = y;
		anticoords->miny = x;
		extents->miny = lat;
	      }
	      if (lat > extents->maxy) {
		coords->maxy = y;
		anticoords->maxy = x;
		extents->maxy = lat;
	      }
	    }
	  }
	}
}

void refine_extents(int exponent, BBox *coords, BBox *anticoords, BBox *extents, projPJ pj_dest, projPJ pj_latlon) {
  double range = pow(10,exponent);
  double step = range * 0.1;
  BBox minxrange = {coords->minx-range, anticoords->minx-range, coords->minx+range, anticoords->minx+range};
  BBox minyrange = {anticoords->miny-range, coords->miny-range, anticoords->miny+range, coords->miny+range};
  BBox maxxrange = {coords->maxx-range, anticoords->maxx-range, coords->maxx+range, anticoords->maxx+range};
  BBox maxyrange = {anticoords->maxy-range, coords->maxy-range, anticoords->maxy+range, coords->maxy+range};

  run_extents(minxrange, step, coords, anticoords, extents, pj_dest, pj_latlon);
  run_extents(minyrange, step, coords, anticoords, extents, pj_dest, pj_latlon);
  run_extents(maxxrange, step, coords, anticoords, extents, pj_dest, pj_latlon);
  run_extents(maxyrange, step, coords, anticoords, extents, pj_dest, pj_latlon);
}

void calc_extents(const char* projection, BBox *coords, BBox *anticoords, BBox *extents) {
	// Initialize
	*extents = (BBox){0, 0, 0, 0};
	*coords = (BBox){0, 0, 0, 0};
	*anticoords = (BBox){0, 0, 0, 0};
	projPJ pj_dest, pj_latlon;

	pj_latlon = pj_init_plus("+proj=latlon +a=1");
	pj_dest = pj_init_plus(projection);
	if (!pj_dest) {
		fprintf(stderr, "Error: Unable to generate output projection object.\n");
		exit(1);
	}

	if (pj_latlon == NULL){ fprintf(stderr,"Failed to init latlon projection!\n"); exit (1);}
        if (pj_dest == NULL){ fprintf(stderr, "Failed to init dest projection!\n"); exit (1);}

	BBox tmpcoords = {-180, -90, 180, 90};

	run_extents(tmpcoords, 1, coords, anticoords, extents, pj_dest, pj_latlon);

	for (int exponent = 1; exponent >= -3; exponent--)
	  refine_extents(exponent, coords, anticoords, extents, pj_dest, pj_latlon);

	pj_free(pj_dest);
	pj_free(pj_latlon);
}

